package myproject
{
	import flash.display.*;
	import flash.events.*;
	import jp.progression.casts.*;
	import jp.progression.*;
	import jp.progression.commands.*;
	import jp.progression.scenes.*;
	import org.papervision3d.cameras.*;
	import org.papervision3d.events.*;
	import org.papervision3d.materials.*;
	import org.papervision3d.materials.utils.*;
	import org.papervision3d.objects.*;
	import org.papervision3d.objects.primitives.*;
	import org.papervision3d.render.*;
	import org.papervision3d.scenes.*;
	import org.papervision3d.view.*;
	
	public class Papervision3DWorld extends CastSprite
	{
		static private const ROUND   :int    = 750;
		static private const PAGE_NUM:int    = 10;
		static private const OBJ_NAME:String = "cube_";
		
		// 3D関連の変数です
		private var wrap:DisplayObject3D;
		private var scene:Scene3D;
		private var viewport:Viewport3D;
		private var camera:Camera3D;
		private var renderer:BasicRenderEngine;
		
		// Progression
		private var prog:Progression;
		
		/*======================================================================*//**
		 * コンストラクター
		 */
		public function Papervision3DWorld( initObject:Object = null )
		{
			super( initObject );
			
			init3d();
			initPosition();
			initContents();
		}
		
		/*======================================================================*//**
		 * ICastObject オブジェクトが AddChild コマンド、または AddChildAt コマンド経由でディスプレイリストに追加された場合に送出されます。
		 * このイベント処理の実行中には、addCommand() メソッド、及び insertCommand() メソッドによるコマンドの同期処理が行えます。
		 */
		protected override function _onCastAdded():void {
			alpha = 0;
			wrap.rotationY = 360;
			
			// 実行したいコマンドを登録します。
			addCommand(
				function():void
				{
					startRendering();
				},
				// []でくくることで並列アニメーション
				[
					new DoTweener(this,
					{
						alpha      : 1,
						transition : "linear",
						time       : 1.5
					}),
					new DoTweener(wrap,
					{
						rotationY  : 0,
						transition : "easeInOutExpo",
						time       : 1.5
					}),
					new DoTweener(camera,
					{
						zoom       : 1,
						transition : "easeInOutExpo",
						time       : 1.5
					})
				]
			);
		}
		
		/*======================================================================*//**
		 * ICastObject オブジェクトが RemoveChild コマンド、または RemoveAllChild コマンド経由でディスプレイリストから削除された場合に送出されます。
		 * このイベント処理の実行中には、addCommand() メソッド、及び insertCommand() メソッドによるコマンドの同期処理が行えます。
		 */
		protected override function _onCastRemoved():void {
			// 実行したいコマンドを登録します。
			addCommand(
				function():void
				{
					stopRendering();
				}
			);
		}
		
		/*======================================================================*//**
		 * 指定したページの番号に遷移します
		 * @param	pageNo	ページ番号
		 */
		public function gotoPage(pageNo:int):void
		{
			new DoTweener(wrap,
			{
				rotationY  : - pageNo / PAGE_NUM * 360,
				transition : "easeOutExpo",
				time       : 1
			}).execute();
		}
		
		/*======================================================================*//**
		 * Papervision3Dの初期化を行います
		 */
		private function init3d():void
		{
			scene    = new Scene3D();
			viewport = new Viewport3D(0, 0, true, true);
			camera   = new Camera3D(60, 10, 500);
			renderer = new BasicRenderEngine();
			
			addChild(viewport);
		}
		
		/*======================================================================*//**
		 * 空間の初期設定を行います
		 */
		private function initPosition():void
		{
			camera.y     = 250;
			camera.z     = 1250;
			camera.lookAt(new DisplayObject3D());
			camera.focus = 400;
			camera.zoom  = 0.1;
		}
		
		/*======================================================================*//**
		 * レンダリングを開始します
		 * @param	e
		 */
		private function startRendering():void
		{
			addEventListener(Event.ENTER_FRAME, enterHandler);
		}
		
		/*======================================================================*//**
		 * レンダリングを停止します
		 * @param	e
		 */
		private function stopRendering():void
		{
			removeEventListener(Event.ENTER_FRAME, enterHandler);
		}
		
		/*======================================================================*//**
		 * コンテンツを初期化します
		 */
		private function initContents():void
		{
			wrap = scene.addChild(new DisplayObject3D());
			
			for (var i:int = 0; i < PAGE_NUM; i++)
			{
				var m:MovieAssetMaterial = new MovieAssetMaterial("Page" + i);
				m.interactive = true;
				
				var o:Cube = new Cube(new MaterialsList( { all:m } ), 320, 50, 240);
				o.name = OBJ_NAME + i;
				o.addEventListener(InteractiveScene3DEvent.OBJECT_CLICK, onClickHandler);
				o.addEventListener(InteractiveScene3DEvent.OBJECT_OVER, onOverHandler);
				o.addEventListener(InteractiveScene3DEvent.OBJECT_OUT, onOutHandler);
				wrap.addChild(o);
				
				o.x = ROUND * Math.sin(i / PAGE_NUM * 360 * Math.PI / 180);
				o.z = ROUND * Math.cos(i / PAGE_NUM * 360 * Math.PI / 180);
				o.rotationY = i / PAGE_NUM * 360;
			}
		}
		
		/*======================================================================*//**
		 * オブジェクトがクリックされたときの処理を設定します
		 * @param	e
		 */
		private function onClickHandler(e:InteractiveScene3DEvent):void 
		{
			var o:DisplayObject3D = e.target as DisplayObject3D;
			var pageNo:int = int(o.name.split(OBJ_NAME)[1]);
			
			prog = getProgressionById("index");
			prog.goto(new SceneId("/index/page" + pageNo));
		}
		
		/*======================================================================*//**
		 * ロールオーバー時の処理です
		 */
		private function onOverHandler(e:InteractiveScene3DEvent):void 
		{
			viewport.containerSprite.buttonMode = true;
		}
		
		/*======================================================================*//**
		 * ロールアウト時の処理です
		 */
		private function onOutHandler(e:InteractiveScene3DEvent):void 
		{
			viewport.containerSprite.buttonMode = false;
		}
		
		/*======================================================================*//**
		 * 毎フレームの処理です
		 * @param	e
		 */
		private function enterHandler(e:Event):void 
		{
			renderer.renderScene(scene, camera, viewport);
		}
		
	}
}